@@ -24,6 +24,13 @@
 #' (ass <- tprAssortment(obj, pars))
 #' plot(obj, assort = ass)
 #'
+#' ## new possibility: variable fixed length assortments
+#' m <- matrix(c(rep(5, length(obj)),
+#'               rep(3, length(obj))), nrow = 2, byrow = TRUE)
+#' pars <- parSort(fixN=3, fixM = m, fixA=10, fixZ=20)
+#' (ass <- tprAssortment(obj, pars))
+#' plot(obj, assort = ass)
+#'
 #' ## deciduous wood
 #' obj <- tprTrees(spp=c(15),
 #'                 Dm=list(40),
@@ -34,6 +41,14 @@
 #' ass <- tprAssortment(obj, pars)
 #' plot(obj, assort=ass)
 #'
+#'
+#' ## new possibility: variable fixed length assortments
+#' m <- matrix(c(rep(5, length(obj)),
+#'               rep(3, length(obj))), nrow = 2, byrow = TRUE)
+#' pars <- parSort(fixN=3, fixM = m, fixA=10, fixZ=20)
+#' (ass <- tprAssortment(obj, pars))
+#' plot(obj, assort = ass)
+#'
 setGeneric("tprAssortment",
            function(obj, pars=NULL, mono=TRUE, Rfn=NULL) standardGeneric("tprAssortment"))
 
@@ -119,27 +134,44 @@ setMethod("tprAssortment", signature = "tprTrees",
               }
 
               ## calculate fix length assortment
-              if(pars@fixN[i] >= 1){
-                actl <- Hfix[i] - acth # actual available length
-                lfix <- pars@fixL[i] + pars@fixA[i] # gross length
-                nfix <- actl %/% lfix
-                ## check for additional fix length, as the last is shorter since
-                ## add-on, does not count on cutting diameter
-                nfix <- nfix + (actl - nfix*lfix) %/% pars@fixL[i]
-                nfix <- as.integer(min(nfix, pars@fixN[i]))
-                hfx <- acth + lfix/2 + (0:(nfix-1))*lfix # height of measurement
-                dfxmr <- tprDiameterCpp(obj[i], Hx=hfx, bark = TRUE)
-                dfxmr <- dfxmr - ifelse(dfxmr >= 20, 0.75, 0.5)
-                mdmfx <- dfxmr - bark(rep(obj@spp[i], length(dfxmr)), Dm = dfxmr, relH = hfx/obj@Ht[i])
-                lfx <- pars@fixL[i]
-                volfx <- pi * (mdmfx/200)^2 * lfx
-                acthfx <- acth + (0:(nfix-1) * lfix)
-                acth <- acth + nfix * lfix
-              } else {
+              if(pars@fixN[i] <= 0){
                 volfx <- 0
                 mdmfx <- 0
                 lfx <- 0
                 acthfx <- 0
+              } else {
+                actl <- Hfix[i] - acth # actual available length
+                if(all(pars@fixM[,i]==0)){
+                  lfx <- pars@fixL[i]
+                  lfix <- pars@fixL[i] + pars@fixA[i] # gross length
+                  nfix <- actl %/% lfix
+                  ## check for additional fix length, as the last is shorter since
+                  ## add-on, does not count on cutting diameter
+                  nfix <- nfix + (actl - nfix*lfix) %/% pars@fixL[i]
+                  nfix <- as.integer(min(nfix, pars@fixN[i]))
+                  lfx <- rep(lfx, nfix)
+                  lfix <- rep(lfix, nfix)
+                } else {
+                  # multiple length in fixed length definition (optimise!)
+                  # need lfx (assortment length), lfix (real length), nfix, hfx
+                  lfx <- fnMultipleFixL(actl, fixN=pars@fixN[i], fixM=pars@fixM[,i], fixA=pars@fixA[i])
+                  lfix <- lfx + pars@fixA[i]
+                  nfix <- length(lfx)
+                }
+                if(nfix > 0){
+                  hfx <- acth + cumsum(lfix) - pars@fixA[i] - 0.5*lfx # height of mdm measurement
+                  dfxmr <- tprDiameterCpp(obj[i], Hx=hfx, bark = TRUE)
+                  dfxmr <- dfxmr - ifelse(dfxmr >= 20, 0.75, 0.5)
+                  mdmfx <- dfxmr - bark(rep(obj@spp[i], length(dfxmr)), Dm = dfxmr, relH = hfx/obj@Ht[i])
+                  volfx <- pi * (mdmfx/200)^2 * lfx
+                  acthfx <- acth + cumsum(lfix) - lfix # stammfuß!
+                  acth <- acth + sum(lfix)
+                } else {
+                  volfx <- 0
+                  mdmfx <- 0
+                  lfx <- 0
+                  acthfx <- 0
+                }
               }
 
               ## calculate sh
@@ -319,7 +351,7 @@ setMethod("tprAssortment", signature = "tprTrees",
               df <- data.frame(tree=i,
                                sort=c("hx", sortfix, "sth", "ab", sortih, "nvd"),
                                height=c(acthxh, acthfx, acthsh, acthab, acthih, acthnvd),
-                               length=c(lxh, rep(lfx, length(volfx)), lsh, lab, rep(lih, length(volih)), lnvd),
+                               length=c(lxh, lfx, lsh, lab, rep(lih, length(volih)), lnvd),
                                mdm=c(mdmxh, mdmfx, mdmsh, mdmab, mdmih, mdmnvd),
                                zdm=NA,
                                vol=c(volxh, volfx, volsh, volab, volih, volnvd))
